import json
from textual.widgets import Input

from ..controller.tasks import (
    get_task,
    get_tasks,
    add_task,
    update_task,
    save_view,
    get_saved_view,
)
from .editor import TableEditor


class TT(TableEditor):

    BINDINGS = [
        # saved views
        ("ctrl+s", "save_view", "save current view"),
        ("ctrl+o", "load_view", "load saved view"),
    ]

    def __init__(self, default_view="default"):
        super().__init__("tasks")
        self.update_item_callback = update_task
        self.add_item_callback = add_task
        self.get_item_callback = get_task

    def _load_db_view(self, name):
        try:
            saved = get_saved_view(name)
            self.filters = json.loads(saved.filters)
            self.sort_string = saved.sort_string
        except Exception:
            self.notify(f"Could not load {name}")

    def action_save_view(self):
        self._show_input("save-view", "default")

    def action_load_view(self):
        self._show_input("load-view", "")

    def on_input_submitted(self, event: Input.Submitted):
        # Override to add save/load view
        if self.mode == "save-view":
            save_view(event.value, filters=self.filters, sort_string=self.sort_string)
            self._hide_input()
            event.prevent_default()
        elif self.mode == "load-view":
            self._load_view(event.value)
            self.refresh_tasks(restore_cursor=False)
            self._hide_input()
            # if event isn't handled here it will bubble to parent
            event.prevent_default()

    def refresh_items(self):
        items = get_tasks(
            self.search_query,
            projects=self._filters_to_list("project"),
            statuses=self._filters_to_list("status"),
            sort=self.sort_string,
        )
        for item in items:
            self.table.add_row(
                *self._db_item_to_row(item),
                key=str(item.id),
            )

    def _filters_to_list(self, key):
        filters = self.filters.get(key)
        if filters:
            return filters.split(",")
        else:
            return None

def run(default_view):
    app = TT(default_view)
    app.run()
